'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = createDevToolsStore;
function createDevToolsStore(onDispatch) {
  var initialState = {
    actionsById: {},
    computedStates: [],
    currentStateIndex: -1,
    monitorState: {},
    nextActionId: 0,
    skippedActionIds: [],
    stagedActionIds: []
  };
  var currentState = [];
  var listeners = [];
  var initiated = false;
  var instance = void 0;

  function getState(id) {
    if (id) return currentState[id] || initialState;
    if (instance) return currentState[instance] || initialState;
    return initialState;
  }

  function getInitialState() {
    return initialState;
  }

  function isSet() {
    return initiated;
  }

  function update() {
    listeners.forEach(function (listener) {
      return listener();
    });
  }

  function setState(state, id, onChanged) {
    var isNew = !currentState[id];
    if (isNew && onChanged) onChanged();
    currentState[id] = state;
    update();
    initiated = true;
  }

  function setInstance(id, toUpdate) {
    instance = id;
    if (toUpdate && instance && instance !== 'auto') update();
  }

  function deleteInstance(id) {
    delete currentState[id];
    instance = Object.keys(currentState)[0];
    update();
  }

  function dispatch(action) {
    if (action.type[0] !== '@') {
      if (action.type === 'JUMP_TO_STATE') {
        var state = getState();
        onDispatch(action, instance, state.computedStates[action.index].state);
        setState(_extends({}, state, { currentStateIndex: action.index }), instance);
      } else onDispatch(action, instance);
    }
    return action;
  }

  function subscribe(listener) {
    listeners.push(listener);

    return function unsubscribe() {
      var index = listeners.indexOf(listener);
      listeners.splice(index, 1);
    };
  }

  return {
    dispatch: dispatch,
    getState: getState,
    subscribe: subscribe,
    liftedStore: {
      dispatch: dispatch,
      getInitialState: getInitialState,
      getState: getState,
      setState: setState,
      setInstance: setInstance,
      deleteInstance: deleteInstance,
      subscribe: subscribe,
      isSet: isSet
    }
  };
}
module.exports = exports['default'];